searchData={"items":[{"type":"module","title":"Puid","doc":"Simple, fast, flexible and efficient generation of probably unique identifiers (`puid`, aka\nrandom strings) of intuitively specified entropy using pre-defined or custom characters.","ref":"Puid.html"},{"type":"module","title":"Overview - Puid","doc":"`Puid` provides fast and efficient generation of random IDs. For the purposes of `Puid`, a random\nID is considered a random string used in a context of uniqueness, that is, random IDs are a bunch\nof random strings that are hopefully unique.\n\nRandom string generation can be thought of as a _transformation_ of some random source of entropy\ninto a string _representation_ of randomness. A general purpose random string library used for\nrandom IDs should therefore provide user specification for each of the following three key\naspects:\n\n#","ref":"Puid.html#module-overview"},{"type":"module","title":"Entropy source - Puid","doc":"What source of randomness is being transformed? `Puid` allows easy specification of the function\n  used for source randomness.\n\n#","ref":"Puid.html#module-entropy-source"},{"type":"module","title":"ID characters - Puid","doc":"What characters are used in the ID? `Puid` provides 16 pre-defined character sets, as well as\n  allows custom character designation, including Unicode\n\n#","ref":"Puid.html#module-id-characters"},{"type":"module","title":"ID randomness - Puid","doc":"What is the resulting “randomness” of the IDs? Note this isn't necessarily the same as the\n  randomness of the entropy source. `Puid` allows explicit specification of ID randomness in an\n  intuitive manner.","ref":"Puid.html#module-id-randomness"},{"type":"module","title":"Examples - Puid","doc":"Creating a random ID generator using `Puid` is a simple as:\n\n```elixir\niex> defmodule(RandId, do: use(Puid))\niex> RandId.generate()\n\"8nGA2UaIfaawX-Og61go5A\"\n```\n\nOptions allow easy and complete control of ID generation.\n\n#","ref":"Puid.html#module-examples"},{"type":"module","title":"Entropy Source - Puid","doc":"`Puid` uses\n[:crypto.strong_rand_bytes/1](https://www.erlang.org/doc/man/crypto.html#strong_rand_bytes-1) as\nthe default entropy source. The `rand_bytes` option can be used to specify any function of the\nform `(non_neg_integer) -> binary` as the source:\n\n```elixir\niex > defmodule(PrngPuid, do: use(Puid, rand_bytes: &:rand.bytes/1))\niex> PrngPuid.generate()\n\"bIkrSeU6Yr8_1WHGvO0H3M\"\n```\n\n#","ref":"Puid.html#module-entropy-source"},{"type":"module","title":"ID Characters - Puid","doc":"By default, `Puid` use the [RFC 4648](https://tools.ietf.org/html/rfc4648#section-5) file system &\nURL safe characters. The `chars` option can by used to specify any of 16 [pre-defined character\nsets](#Chars) or custom characters, including Unicode:\n\n```elixir\niex> defmodule(HexPuid, do: use(Puid, chars: :hex))\niex> HexPuid.generate()\n\"13fb81e35cb89e5daa5649802ad4bbbd\"\n\niex> defmodule(DingoskyPuid, do: use(Puid, chars: \"dingosky\"))\niex> DingoskyPuid.generate()\n\"yiidgidnygkgydkodggysonydodndsnkgksgonisnko\"\n\niex> defmodule(DingoskyUnicodePuid, do: use(Puid, chars: \"dîñgø$kyDÎÑGØßK¥\", total: 2.5e6, risk: 1.0e15))\niex> DingoskyUnicodePuid.generate()\n\"øßK$ggKñø$dyGîñdyØøØÎîk\"\n\n```\n\n#","ref":"Puid.html#module-id-characters"},{"type":"module","title":"ID Randomness - Puid","doc":"Generated IDs have 128-bit entropy by default. `Puid` provides a simple, intuitive way to specify\nID randomness by declaring a `total` number of possible IDs with a specified `risk` of a repeat in\nthat many IDs:\n\nTo generate up to _10 million_ random IDs with _1 in a trillion_ chance of repeat:\n\n```elixir\niex> defmodule(MyPuid, do: use(Puid, total: 10.0e6, risk: 1.0e15))\niex> MyPuid.generate()\n\"T0bFZadxBYVKs5lA\"\n```\n\nThe `bits` option can be used to directly specify an amount of ID randomness:\n\n```elixir\niex> defmodule(Token, do: use(Puid, bits: 256, chars: :hex_upper))\niex> Token.generate()\n\"6E908C2A1AA7BF101E7041338D43B87266AFA73734F423B6C3C3A17599F40F2A\"\n```","ref":"Puid.html#module-id-randomness"},{"type":"module","title":"Module API - Puid","doc":"`Puid` modules have two functions:\n\n```elixir\niex> defmodule(AlphanumId, do: use(Puid, total: 10.0e06, risk: 1.0e15, chars: :alphanum))\n```\n\n**`generate/0`**\n\nGenerates a **`puid`**\n\n```elixir\niex> AlphanumId.generate()\n\"UKQHTmvASwyhcwGNA\"\n```\n\n**`info/0`**\n\n  Returns a `Puid.Info` structure consisting of\n\n  - Name of pre-defined `Puid.Chars` or `:custom`\n  - Source characters\n  - Entropy bits\n    - May be larger than the specified `bits` since it is a multiple of the entropy bits per\n      character\n  - Entropy bits per character\n  - Entropy representation efficiency\n    - Ratio of the **`puid`** entropy to the bits required for string representation\n  - **`puid`** string length\n  - Entropy source function\n\n```elixir\niex> AlphanumId.info()\n %Puid.Info{\n   char_set: :alphanum,\n   characters: \"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789\",\n   entropy_bits: 101.22,\n   entropy_bits_per_char: 5.95,\n   ere: 0.74,\n   length: 17,\n   rand_bytes: &:crypto.strong_rand_bytes/1\n }\n```","ref":"Puid.html#module-module-api"},{"type":"module","title":"Puid.Chars","doc":"Pre-defined character sets for use when creating `Puid` modules.","ref":"Puid.Chars.html"},{"type":"module","title":"Example - Puid.Chars","doc":"iex> defmodule(AlphanumId, do: use(Puid, chars: :alphanum))","ref":"Puid.Chars.html#module-example"},{"type":"module","title":"Pre-defined Chars - Puid.Chars","doc":"### :alpha\nUpper/lower case alphabet\n\n```none\nABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz\n```\nbits per character: `5.7`\n\n### :alpha_lower\nLower case alphabet\n```none\nabcdefghijklmnopqrstuvwxyz\n```\nbits per character: `4.7`\n\n### :alpha_upper\nUpper case alphabet\n```none\nABCDEFGHIJKLMNOPQRSTUVWXYZ\n```\nbits per character: `4.7`\n\n### :alphanum\nUpper/lower case alphabet and numbers\n```none\nABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789\n```\nbits per character: `5.95`\n\n### :alphanum_lower\nLower case alphabet and numbers\n```none\nabcdefghijklmnopqrstuvwxyz0123456789\n```\nbits per character: `5.17`\n\n### :alphanum_upper\nUpper case alphabet and numbers\n```none\nABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789\n```\nbits per character: `5.17`\n\n### :base16\n[RFC 4648](https://tools.ietf.org/html/rfc4648#section-8) base16 character set\n```\n0123456789ABCDEF\n```\nbits per character: `4`\n\n### :base32\n[RFC 4648](https://tools.ietf.org/html/rfc4648#section-6) base32 character set\n```none\nABCDEFGHIJKLMNOPQRSTUVWXYZ234567\n```\nbits per character: `5`\n\n### :base32_hex\n[RFC 4648](https://tools.ietf.org/html/rfc4648#section-7) base32 extended hex character set\nwith lowercase letters\n```none\n0123456789abcdefghijklmnopqrstuv\n```\nbits per character: `5`\n\n### :base32_hex_upper\n[RFC 4648](https://tools.ietf.org/html/rfc4648#section-7) base32 extended hex character set\n```none\n0123456789ABCDEFGHIJKLMNOPQRSTUV\n```\nbits per character: `5`\n\n### :crockford32\n[Crockford 32](https://www.crockford.com/base32.html)\n```none\n0123456789ABCDEFGHJKMNPQRSTVWXYZ\n```\n\n### :decimal\nDecimal digits\n```none\n0123456789\n```\nbits per character: `3.32`\n\n### :hex\nLowercase hexadecimal\n```none\n0123456789abcdef\n```\nbits per character: `4`\n\n### :hex_upper\nUppercase hexadecimal\n```none\n0123456789ABCDEF\n```\nbits per character: `4`\n\n### :safe_ascii\nASCII characters from `?!` to `?~`, minus backslash, backtick, single-quote and double-quote\n\n```none\n`!#$%&()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[]^_\u0007bcdefghijklmnopqrstuvwxyz{|}~`\n```\nbits per character: `6.49`\n\n### :safe32\nStrings that don't look like English words and are easier to parse visually\n```none\n2346789bdfghjmnpqrtBDFGHJLMNPQRT\n```\n  - remove all upper and lower case vowels (including y)\n  - remove all numbers that look like letters\n  - remove all letters that look like numbers\n  - remove all letters that have poor distinction between upper and lower case values\n\nbits per character: `6.49`\n\n### :safe64\n[RFC 4648](https://tools.ietf.org/html/rfc4648#section-5) file system and URL safe character set\n```none\nABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_\n```\nbits per character: `6`\n\n### :symbol\n:safe_ascii characters not in :alphanum\n\n```none\n`!#$%&()*+,-./:;<=>?@[]^_{|}~`\n```\nbits per character: `4.81`\n\n### :wordSafe32\nStrings that don't look like English words\n```none\n23456789CFGHJMPQRVWXcfghjmpqrvwx\n```\nOrigin unknown\n\nbits per character: `6.49`","ref":"Puid.Chars.html#module-pre-defined-chars"},{"type":"function","title":"Puid.Chars.charlist/1","doc":"`charlist` for a pre-defined `Puid.Chars`, a String.t() or a charlist.\n\nThe characters for either String.t() or charlist types must be unique, have more than one\ncharacter, and not be invalid ascii.","ref":"Puid.Chars.html#charlist/1"},{"type":"function","title":"Example - Puid.Chars.charlist/1","doc":"iex> Puid.Chars.charlist(:safe32)\n    {:ok, ~c\"2346789bdfghjmnpqrtBDFGHJLMNPQRT\"}\n\n    iex> Puid.Chars.charlist(\"dingosky\")\n    {:ok, ~c\"dingosky\"}\n\n    iex> Puid.Chars.charlist(\"unique\")\n    {:error, \"Characters not unique\"}","ref":"Puid.Chars.html#charlist/1-example"},{"type":"function","title":"Puid.Chars.charlist!/1","doc":"Same as `charlist/1` but either returns __charlist__ or raises a `Puid.Error`","ref":"Puid.Chars.html#charlist!/1"},{"type":"function","title":"Example - Puid.Chars.charlist!/1","doc":"iex> Puid.Chars.charlist!(:safe32)\n    ~c\"2346789bdfghjmnpqrtBDFGHJLMNPQRT\"\n\n    iex> Puid.Chars.charlist!(\"dingosky\")\n    ~c\"dingosky\"\n\n    iex> Puid.Chars.charlist!(\"unique\")\n    # (Puid.Error) Characters not unique","ref":"Puid.Chars.html#charlist!/1-example"},{"type":"type","title":"Puid.Chars.puid_chars/0","doc":"Chars can be designated by a pre-defined atom, a binary or a charlist","ref":"Puid.Chars.html#t:puid_chars/0"},{"type":"type","title":"Puid.Chars.puid_encoding/0","doc":"Character encoding scheme. `:ascii` encoding uses cross-product character pairs.","ref":"Puid.Chars.html#t:puid_encoding/0"},{"type":"module","title":"Puid.Entropy","doc":"[Entropy](https://en.wikipedia.org/wiki/Entropy_(information_theory)) related calculations\n\nThe implementation is based on mathematical approximations to the solution of what is often\nreferred to as the [Birthday\nProblem](https://en.wikipedia.org/wiki/Birthday_problem#Calculating_the_probability).","ref":"Puid.Entropy.html"},{"type":"function","title":"Puid.Entropy.bits/2","doc":"Entropy bits for generating a `total` number of events with the given `risk` of repeat\n\nThe total size of the event pool is 2 bits .","ref":"Puid.Entropy.html#bits/2"},{"type":"function","title":"Example - Puid.Entropy.bits/2","doc":"iex> Puid.Entropy.bits(10.0e6, 1.0e12)\n    85.37013046707142","ref":"Puid.Entropy.html#bits/2-example"},{"type":"function","title":"Puid.Entropy.bits_for_len/2","doc":"Entropy bits for a binary of length `len` comprised of `chars` characters.\n\n`chars` must be valid as per `Chars.charlist/1`.","ref":"Puid.Entropy.html#bits_for_len/2"},{"type":"function","title":"Example - Puid.Entropy.bits_for_len/2","doc":"iex> Puid.Entropy.bits_for_len(:alphanum, 14)\n    {:ok, 83}\n\n    iex> Puid.Entropy.bits_for_len('dingosky', 14)\n    {:ok, 42}","ref":"Puid.Entropy.html#bits_for_len/2-example"},{"type":"function","title":"Puid.Entropy.bits_for_len!/2","doc":"Same as `Puid.Entropy.bits_for_len/2` but either returns __bits__ or raises a\n`Puid.Error`","ref":"Puid.Entropy.html#bits_for_len!/2"},{"type":"function","title":"Example - Puid.Entropy.bits_for_len!/2","doc":"iex> Puid.Entropy.bits_for_len!(:alphanum, 14)\n    83\n\n    iex> Puid.Entropy.bits_for_len!(\"dingosky\", 14)\n    42","ref":"Puid.Entropy.html#bits_for_len!/2-example"},{"type":"function","title":"Puid.Entropy.bits_per_char/1","doc":"Entropy bits per `chars` character.\n\n`chars` must be valid as per `Chars.charlist/1`.","ref":"Puid.Entropy.html#bits_per_char/1"},{"type":"function","title":"Example - Puid.Entropy.bits_per_char/1","doc":"iex> Puid.Entropy.bits_per_char(:alphanum)\n    {:ok, 5.954196310386875}\n\n    iex> Puid.Entropy.bits_per_char(\"dingosky\")\n    {:ok, 3.0}","ref":"Puid.Entropy.html#bits_per_char/1-example"},{"type":"function","title":"Puid.Entropy.bits_per_char!/1","doc":"Same as `bits_per_char/1` but either returns __bits__ or raises a `Puid.Error`","ref":"Puid.Entropy.html#bits_per_char!/1"},{"type":"function","title":"Example - Puid.Entropy.bits_per_char!/1","doc":"iex> Puid.Entropy.bits_per_char!(:alphanum)\n     5.954196310386875\n\n     Puid.Entropy.bits_per_char!(\"dingosky\")\n     3.0","ref":"Puid.Entropy.html#bits_per_char!/1-example"},{"type":"function","title":"Puid.Entropy.len_for_bits/2","doc":"Length needed for a string generated from `chars` to have entropy `bits`.\n\n`chars` must be valid as per `Chars.charlist/1`.","ref":"Puid.Entropy.html#len_for_bits/2"},{"type":"function","title":"Example - Puid.Entropy.len_for_bits/2","doc":"iex> Puid.Entropy.len_for_bits(:alphanum, 128)\n    {:ok, 22}\n\n    iex> Puid.Entropy.len_for_bits(\"dingosky\", 128)\n    {:ok, 43}","ref":"Puid.Entropy.html#len_for_bits/2-example"},{"type":"function","title":"Puid.Entropy.len_for_bits!/2","doc":"Same as `Puid.Entropy.len_for_bits/2` but either returns __len__ or raises a\n`Puid.Error`","ref":"Puid.Entropy.html#len_for_bits!/2"},{"type":"function","title":"Example - Puid.Entropy.len_for_bits!/2","doc":"iex> Puid.Entropy.len_for_bits!(:alphanum, 128)\n    22\n\n    iex> Puid.Entropy.len_for_bits!('dingosky', 128)\n    43","ref":"Puid.Entropy.html#len_for_bits!/2-example"},{"type":"type","title":"Puid.Entropy.puid_chars/0","doc":"","ref":"Puid.Entropy.html#t:puid_chars/0"},{"type":"module","title":"Puid.Info","doc":"Information regarding Puid module parameterization\n\nThe `Puid.Info` struct has the following fields:\n\n| Field | Description |\n| ----- | ----------- |\n| char_set | pre-defined `Puid.Chars` atom or :custom |\n| characters | source characters |\n| entropy_bits | entropy bits for generated **puid** |\n| entropy_bits_per_char | entropy bits per character for generated **puid**s |\n| ere | **puid** entropy string representation efficiency |\n| length | **puid** string length |\n| rand_bytes | entropy source function |\n\n```elixir\niex> defmodule(CustomId, do: use(Puid, total: 1.0e04, risk: 1.0e12, chars: \"thequickbrownfxjmpsvlazydg\"))\niex> CustomId.info()\n%Puid.Info{\n  char_set: :custom,\n  characters: \"thequickbrownfxjmpsvlazydg\",\n  entropy_bits: 65.81,\n  entropy_bits_per_char: 4.7,\n  ere: 0.59,\n  length: 14,\n  rand_bytes: &:crypto.strong_rand_bytes/1\n}\n```","ref":"Puid.Info.html"},{"type":"exception","title":"Puid.Error","doc":"Raised when defining a Puid module with invalid options","ref":"Puid.Error.html"},{"type":"type","title":"Puid.Error.t/0","doc":"`Puid.Error` type","ref":"Puid.Error.html#t:t/0"}],"content_type":"text/markdown"}